"""Feature testing."""
import creopyson
import pytest
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None, mk_creoson_post_list


def test_file_assemble(mk_creoson_post_dict):
    """Test assemble."""
    c = creopyson.Client()
    result = c.file_assemble(
        "file",
        dirname="dirname",
        generic="generic",
        into_asm="target",
        path="path",
        ref_model="model",
        transform={"fake transform":122},
        constraints={"fake constraint":122},
        package_assembly=True,
        walk_children=True,
        assemble_to_root=True,
        suppress=True
    )
    assert isinstance(result, (dict))


def test_file_backup(mk_creoson_post_None):
    """Test backup."""
    c = creopyson.Client()
    result = c.file_backup(
        "target",
        file_="file"
    )
    assert result is None


def test_file_close_window(mk_creoson_post_None):
    """Test close_window."""
    c = creopyson.Client()
    result = c.file_close_window(file_="file")
    assert result is None


def test_file_display(mk_creoson_post_None):
    """Test display."""
    c = creopyson.Client()
    result = c.file_display("file", activate=True)
    assert result is None


def test_file_erase(mk_creoson_post_None):
    """Test erase."""
    c = creopyson.Client()
    result = c.file_erase(
        file_="file",
        erase_children=True
    )
    assert result is None
    result = c.file_erase(
        file_=["file", "other file"]
    )
    assert result is None


def test_file_erase_not_displayed(mk_creoson_post_None):
    """Test erase_not_displayed."""
    c = creopyson.Client()
    result = c.file_erase_not_displayed()
    assert result is None


def test_file_exists(mk_creoson_post_dict):
    """Test exists."""
    c = creopyson.Client()
    result = c.file_exists(file_="file")
    assert result is True


def test_file_get_active(mk_creoson_post_dict):
    """Test get_active."""
    c = creopyson.Client()
    result = c.file_get_active()
    assert isinstance(result, (dict))


def test_file_get_fileinfo(mk_creoson_post_dict):
    """Test get_fileinfo."""
    c = creopyson.Client()
    result = c.file_get_fileinfo(file_="file")
    assert isinstance(result, (dict))


def test_file_get_length_units(mk_creoson_post_dict):
    """Test get_length_units."""
    c = creopyson.Client()
    result = c.file_get_length_units(file_="file")
    assert isinstance(result, (str))


def test_file_get_mass_units(mk_creoson_post_dict):
    """Test get_mass_units."""
    c = creopyson.Client()
    result = c.file_get_mass_units(file_="file")
    assert isinstance(result, (str))


def test_file_get_transform(mk_creoson_post_dict):
    """Test get_transform."""
    c = creopyson.Client()
    result = c.file_get_transform(
        asm="asm",
        path="path",
        csys="csys"
    )
    assert isinstance(result, (dict))


def test_file_has_instances(mk_creoson_post_dict):
    """Test has_instances."""
    c = creopyson.Client()
    result = c.file_has_instances(file_="file")
    assert result is True


def test_file_is_active(mk_creoson_post_dict):
    """Test is_active."""
    c = creopyson.Client()
    result = c.file_is_active(file_="file")
    assert result is True


def test_file_list(mk_creoson_post_dict):
    """Test list."""
    c = creopyson.Client()
    result = c.file_list(file_="file")
    assert isinstance(result, (list))
    result = c.file_list(file_=["file", "other file"])
    assert isinstance(result, (list))


def test_file_list_instances(mk_creoson_post_dict):
    """Test list_instances."""
    c = creopyson.Client()
    result = c.file_list_instances(file_="file")
    assert isinstance(result, (dict))


def test_file_list_simp_reps(mk_creoson_post_dict):
    """Test list_simp_reps."""
    c = creopyson.Client()
    result = c.file_list_simp_reps(
        file_="file",
        rep="rep"
    )
    assert isinstance(result, (dict))


def test_file_massprops(mk_creoson_post_dict):
    """Test massprops."""
    c = creopyson.Client()
    result = c.file_massprops(file_="file")
    assert isinstance(result, (dict))


def test_file_open(mk_creoson_post_dict):
    """Test open."""
    c = creopyson.Client()
    result = c.file_open(
        file_="file",
        dirname="dirname",
        generic="generic",
        display=True,
        activate=True,
        new_window=True,
        regen_force=True
    )
    assert isinstance(result, (dict))
    result = c.file_open(
        file_=["file", "other file"],
    )
    assert isinstance(result, (dict))


def test_file_open_errors(mk_creoson_post_dict):
    """Test open_errors."""
    c = creopyson.Client()
    result = c.file_open_errors(file_="file")
    assert result is True


def test_file_postregen_relations_get(mk_creoson_post_dict):
    """Test postregen_relations_get."""
    c = creopyson.Client()
    result = c.file_postregen_relations_get(file_="file")
    assert isinstance(result, (list))


def test_file_postregen_relations_set(mk_creoson_post_None):
    """Test postregen_relations_set."""
    c = creopyson.Client()
    result = c.file_postregen_relations_set(
        file_="file",
        relations=["relation"]
    )
    assert result is None


def test_file_refresh(mk_creoson_post_None):
    """Test refresh."""
    c = creopyson.Client()
    result = c.file_refresh(file_="file")
    assert result is None


def test_file_regenerate(mk_creoson_post_None):
    """Test regenerate."""
    c = creopyson.Client()
    result = c.file_regenerate(
        file_="file",
        display=True
    )
    assert result is None
    result = c.file_regenerate(
        file_=["file", "other file"]
    )
    assert result is None


def test_file_relations_get(mk_creoson_post_dict):
    """Test relations_get."""
    c = creopyson.Client()
    result = c.file_relations_get(file_="file")
    assert isinstance(result, (list))


def test_file_relations_set(mk_creoson_post_None):
    """Test relations_set."""
    c = creopyson.Client()
    result = c.file_relations_set(
        file_="file",
        relations=["relation"]
    )
    assert result is None


def test_file_rename(mk_creoson_post_dict):
    """Test rename."""
    c = creopyson.Client()
    result = c.file_rename(
        "new_name",
        file_="file",
        onlysession=True
    )
    assert isinstance(result, (str))


def test_file_repaint(mk_creoson_post_None):
    """Test repaint."""
    c = creopyson.Client()
    result = c.file_repaint(file_="file")
    assert result is None


def test_file_save(mk_creoson_post_None):
    """Test save."""
    c = creopyson.Client()
    result = c.file_save("file")
    assert result is None
    result = c.file_save(["file", "other file"])
    assert result is None


def test_file_set_length_units(mk_creoson_post_None):
    """Test set_length_units."""
    c = creopyson.Client()
    result = c.file_set_length_units(
        "mm",
        file_="file",
        convert=True
    )
    assert result is None


def test_file_set_mass_units(mk_creoson_post_None):
    """Test set_mass_units."""
    c = creopyson.Client()
    result = c.file_set_mass_units(
        "g",
        file_="file",
        convert=True
    )
    assert result is None



"""Creo module."""


def cd(client, dirname):
    r"""Change Creo's working directory.

    You can use absolute path or relative:
    "C:\\My Workdir\\"
    "..\\Other_directory\\"

    Args:
        client (obj): creopyson Client.
        dirname (str): New directory name.

    Returns:
        (str): Name of new working directory.

    """
    data = {"dirname": dirname}
    return client._creoson_post("creo", "cd", data, "dirname")


def delete_files(client, filename=None, dirname=None):
    """Delete files from a directory working directory.

    Args:
        client (obj):
            creopyson Client.
        filename (str or list:str, optional):
            File name filter or list of file names.
            if blank all files will be deleted.
            (wildcards_allowed: True). Defaults to None.
        dirname (str, optional):
            Directory name. Defaults is Creo's current working directory.

    Returns:
        (list:str): List of deleted files.

    """
    data = {}
    if filename:
        if isinstance(filename, (list)):
            data["filenames"] = filename
        else:
            data["filename"] = str(filename)
    if dirname:
        data["dirname"] = dirname
    return client._creoson_post("creo", "delete_files", data, "filelist")


def get_config(client, name):
    """Get the value of a Creo config option.

    Args:
        client (obj): creopyson Client.
        name (str): Option name.

    Raises:
        Warning: error message from creoson.

    Returns:
        (list:str):
            List of option values (some options can have multiple values).

    """
    data = {"name": name}
    return client._creoson_post("creo", "get_config", data, "values")


def get_std_color(client, color_type):
    """Get one of Creo's standard colors.

    Args:
        client (obj):
            creopyson Client.
        color_type (str):
            Color type.
            Valid values: letter, highlight, drawing, background, half_tone,
            edge_highlight, dimmed, error, warning, sheetmetal, curve,
            presel_highlight, selected, secondary_selected, preview,
            secondary_preview, datum, quilt.

    Returns:
        (dict):
            red (int): Red value (0-255)
            green (int): Green value (0-255)
            blue (int): Blue value (0-255)

    """
    data = {"color_type": color_type}
    return client._creoson_post("creo", "get_std_color", data)


def list_dirs(client, dirname=None):
    """List subdirectories of Creo's current working directory.

    Args:
        client (obj):
            creoopyson Client.
        dirname (str, optional):
            Directory name filter (wildcards_allowed: True).
            Defaults: All subdirectories are listed.

    Returns:
        (list:str): List of subdirectories

    """
    data = {"dirname": "*"}
    if dirname:
        data["dirname"] = dirname
    return client._creoson_post("creo", "list_dirs", data, "dirlist")


def list_files(client, filename=None):
    """List files in Creo's current working directory.

    Args:
        client (obj):
            creopyson Client.
        filename (str, optional):
            File name filter (wildcards_allowed: True).
            Defaults: all files are listed.

    Returns:
        (list:int): List of files.

    """
    data = {"filename": "*"}
    if filename:
        data["filename"] = filename
    return client._creoson_post("creo", "list_files", data, "filelist")


def mkdir(client, dirname):
    """Create a new directory.

    Args:
        client (obj): creopyson Client.
        dirname (str): New directory name.

    Returns:
        (str): Full name of new working directory.

    """
    data = {"dirname": dirname}
    return client._creoson_post("creo", "mkdir", data, "dirname")


def pwd(client):
    """Return Creo's current working directory.

    Args:
        client (obj): creopyson Client.

    Returns:
        (str): Full name of working directory.

    """
    return client._creoson_post("creo", "pwd", key_data="dirname")


def rmdir(client, dirname):
    """Delete a directory.

    Args:
        client (obj): creopyson Client.
        dirname (str): Directory name to delete.

    Returns:
        None.

    """
    data = {"dirname": dirname}
    return client._creoson_post("creo", "rmdir", data)


def set_config(client, name, value, ignore_errors=None):
    """Set a Creo config option.

    Args:
        client (obj):
            creopyson Client.
        name (str):
            Option name.
        value (str):
            New option value.
        ignore_errors (boolean, optional):
            Whether to ignore errors that might occur when setting the config
            option. Defaults is False.

    Returns:
        None.

    """
    data = {
        "name": name,
        "value": value,
        "ignore_errors": False
    }
    if ignore_errors:
        data["ignore_errors"] = ignore_errors
    return client._creoson_post("creo", "set_config", data)


def set_std_color(client, color_type, red, green, blue):
    """Set one of Creo's standard colors.

    Args:
        client (obj):
            creopyson Client.
        color_type (str):
            Color type.
            Valid values: letter, highlight, drawing, background, half_tone,
            edge_highlight, dimmed, error, warning, sheetmetal, curve,
            presel_highlight, selected, secondary_selected, preview,
            secondary_preview, datum, quilt.
        red (int):
            Red value (0-255).
        green (int):
            Green value (0-255).
        blue (int):
            Blue value (0-255).

    Returns:
        None.

    """
    data = {
        "color_type": color_type,
        "red": red,
        "green": green,
        "blue": blue
    }
    return client._creoson_post("creo", "rmdir", data)
    # TODO: convert RGB to a tuple or hexa color?



"""File module."""


def assemble(
    client,
    file_,
    dirname=None,
    generic=None,
    into_asm=None,
    path=None,
    ref_model=None,
    transform=None,
    constraints=None,
    package_assembly=None,
    walk_children=None,
    assemble_to_root=None,
    suppress=None
):
    """Assemble a component into an assembly.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str):
            File name component.
        dirname (str, optional):
            Diretory name. Defaults is Creo's current working directory.
        generic (str, optional):
            Generic model name (if file name represents an instance).
            Defaults is generic model name (if file name represents an
            instance).
        into_asm (str, optional):
            Target assembly. Defaults is currently active model.
        path (list:int, optional):
            Path to a component that the new part will be constrained to.
            Defaults to None.
        ref_model (str, optional):
            Reference model that the new part will be constrained to;
            only used if path is not given.  If there are multiple of this
            model in the assembly, the component will be assembled multiple
            times, once to each occurrence. Defaults to None.
        transform (obj:JLTransform, optional):
            Transform structure for the initial position and orientation of
            the new component; only used if there are no constraints, or for
            certain constraint types. Defaults to None.
        constraints (obj_array:JLConstraint, optional):
            Assembly constraints. Defaults to None.
        package_assembly (boolean, optional):
            Whether to package the component to the assembly; only used if
            there are no constraints specified. Defaults is If there are no
            constraints, then the user will be prompted to constrain the
            component through the Creo user interface.
        walk_children (boolean, optional):
            Whether to walk into subassemblies to find reference models to
            constrain to. Defaults to None.
        assemble_to_root (boolean, optional):
            Whether to always assemble to the root assembly, or assemble to
            the subassembly containing the reference path/model.
            Defaults to None.
        suppress (boolean, optional):
            Whether to suppress the components immediately after assembling
            them. Defaults to None.

    Returns:
        (dict):
            dirname (str):
                Directory name of component.
            files (list:str):
                File name of component.
            revision (int):
                Revision of file that was opened; if more than one
                file was opened, this field is not returned.
            featureid (int):
                Last Feature ID of component after assembly.

    """
    data = {"file": file_}
    if dirname:
        data["dirname"] = dirname
    if generic:
        data["generic"] = generic
    if into_asm:
        data["into_asm"] = into_asm
    if path:
        data["path"] = path
    if ref_model:
        data["ref_model"] = ref_model
    if transform:
        data["transform"] = transform
    if constraints:
        data["constraints"] = constraints
    if package_assembly:
        data["package_assembly"] = package_assembly
    if walk_children:
        data["walk_children"] = walk_children
    if assemble_to_root:
        data["assemble_to_root"] = assemble_to_root
    if suppress:
        data["suppress"] = suppress
    return client._creoson_post("file", "assemble", data)


def backup(client, target_dir, file_=None):
    """Backup a model.

    Args:
        client (obj):
            creopyson Client.
        target_dir (str):
            Target directory name.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        None

    """
    data = {"target_dir": target_dir}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "backup", data)


def close_window(client, file_=None):
    """Close the window containing a model.

    Args:
        client (obj):
            creopyson object.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        None

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "close_window", data)


def display(client, file_, activate=None):
    """Display a model in a window.

    Args:
        client (obj):
            creopyson object.
        `file_` (str):
            File name
        activate (boolean, optional):
            Activate the model after displayong. Defaults is False.

    Returns:
        None

    """
    data = {"file": file_}
    if activate:
        data["activate"] = activate
    return client._creoson_post("file", "display", data)


def erase(client, file_=None, erase_children=None):
    """Erase one or more models from memory.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str|list:str, optional):
            File name or List of file names;
            (Wildcards allowed: True).
            if emptyall models in memory are erased.
        erase_children (boolean, optional):
            Erase children of the models too. Defaults is False.

    Returns:
        None

    """
    data = {}
    if file_:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    if erase_children:
        data["erase_children"] = erase_children
    return client._creoson_post("file", "erase", data)


def erase_not_displayed(client):
    """Erase all non-displayed models from memory.

    Args:
        client (obj): creopyson Client.

    Returns:
        None

    """
    return client._creoson_post("file", "erase_not_displayed")


def exists(client, file_):
    """Check whether a model exists in memory.

    Args:
        client (obj): creopyson Client.
        `file_` (str): File name.

    Returns:
        (Boolean): Whether the file is open in Creo.

    """
    data = {"file": file_}
    return client._creoson_post("file", "exists", data, "exists")


def get_active(client):
    """Get the active model from Creo.

    Args:
        client (obj): creopyson Client.

    Returns:
        (dict):
            dirname (str): Directory name of current model.
            file (str): File name of current model.

    """
    return client._creoson_post("file", "get_active")


def get_fileinfo(client, file_=None):
    """Open one or more files in memory or from the drive.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (dict):
            dirname (str): Directory name of the file.
            file (str): File name.
            revision (int): Revision number of file.

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "get_fileinfo", data)


def get_length_units(client, file_=None):
    """Get the current length units for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (str): Length units.

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "get_length_units", data, "units")


def get_mass_units(client, file_=None):
    """Get the current mass units for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (str): mass units.

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "get_mass_units", data, "units")


def get_transform(client, asm=None, path=None, csys=None):
    """Get the 3D transform for a component in an assembly.

    Args:
        client (obj):
            creopyson Client
        asm (str, optional):
            Assembly name. Defaults is currently active model.
        path (list:int, optional):
            Path to a component in the assembly.
            Defaults is the transform is calculated for the assembly itself.
        csys (str, optional):
            Coordinate system on the component to calculate the transform for.
            Defaults is the component's default coordinate system.

    Returns:
        (obj:JLTransform): The 3D transform from the assembly to
        the component's coordinate system.

    """
    data = {}
    if asm:
        data["asm"] = asm
    if path:
        data["path"] = path
    if csys:
        data["csys"] = csys
    return client._creoson_post("file", "get_transform", data, "transform")


def has_instances(client, file_=None):
    """Check whether a model has a family table.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (boolean): Whether the file has a family table.

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "has_instances", data, "exists")


def is_active(client, file_):
    """Check whether a model is the active model.

    Args:
        client (obj): creopyson Client.
        `file_` (str): File name.

    Returns:
        (boolean): Whether the file is the currently active model.

    """
    data = {"file": file_}
    return client._creoson_post("file", "is_active", data, "active")


def list_(client, file_=None):
    """Get a list of files in the current Creo session that match patterns.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str|list:str, optional):
            File name or List of file names;

    Returns:
        (list:str) List of files.

    """
    data = {}
    if file_:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    return client._creoson_post("file", "is_active", data, "files")


def list_instances(client, file_=None):
    """List instances in a model's family table.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (dict):
            dirname (str): Directory name of the file.
            generic (str): Generic name.
            files (list:str): List of model names in the table.

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "list_instances", data)


def list_simp_reps(client, file_=None, rep=None):
    """List simplified reps in a model.

    Args:
        client (obj):
            creopyson Client
        `file_` (str, optional):
            File name. Defaults is currently active model.
        rep (str, optional):
            Simplified rep name pattern (wildcards_allowed: True).
            Defaults is all simplified reps.

    Returns:
        (dict):
            rep (str): Simplified rep name.
            reps (list:str): Simplified reps names.

    """
    data = {}
    if file_:
        data["file"] = file_
    if rep:
        data["rep"] = rep
    return client._creoson_post("file", "list_simp_reps", data)


def massprops(client, file_=None):
    """Get mass property information about a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (dict):
            volume (float): Model volume.
            mass (float): Model mass.
            density (float): Model density.
            surface_area (float): Model surface area.

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "massprops", data)


def open_(
    client,
    file_,
    dirname=None,
    generic=None,
    display=None,
    activate=None,
    new_window=None,
    regen_force=None
):
    """Open one or more files in memory or from the drive.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str|list:str):
            File name or List of file names;
        dirname (str, optional):
            Directory name. Defaults is Creo's current working directory.
        generic (str, optional):
            Generic model name (if file name represents an instance).
            Defaults to None.
        display (boolean, optional):
            Display the model after opening. Defaults is False.
        activate (boolean, optional):
            Activate the model after opening. Defaults is False.
        new_window (boolean, optional):
            Open model in a new window. Defaults is False.
        regen_force (boolean, optional):
            Force regeneration after opening. Defaults is False.

    Returns:
        (dict):
            dirname (str):
                Directory name of opened file(s).
            files (list:str):
                File names that were opened.
            revision (int):
                Revision of file that was opened;
                if more than one file was opened, this field is not returned.

    """
    data = {}
    if file_:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    if dirname:
        data["dirname"] = dirname
    if generic:
        data["generic"] = generic
    if display:
        data["display"] = display
    if activate:
        data["activate"] = activate
    if new_window:
        data["new_window"] = new_window
    if regen_force:
        data["regen_force"] = regen_force
    return client._creoson_post("file", "open", data)
    # TODO


def open_errors(client, file_=None):
    """Check whether Creo errors have occurred opening a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (boolean): Whether errors exist in Creo.

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "open_errors", data, "errors")


def postregen_relations_get(client, file_=None):
    """Get post-regeneration relations for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (list:str): Exported relations text, one entry per line.

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post(
        "file", "postregen_relations_get", data, "relations")


def postregen_relations_set(client, file_=None, relations=None):
    """Set post-regeneration relations for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.
        relations (list:str, optional):
            Relations text to import, one line per entry.
            Clear the relations if missing.

    Retunrs:
        None

    """
    data = {}
    if file_:
        data["file"] = file_
    if relations:
        data["relations"] = relations
    return client._creoson_post("file", "postregen_relations_set", data)


def refresh(client, file_=None):
    """Refresh the window containing a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        None

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "refresh", data)


def regenerate(client, file_=None, display=None):
    """Regenerate one or more models.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str|list:str, optional):
            File name or List of file names;
            Defaults is currently active model
        display (boolean, optional):
            Display the model before regenerating. Defaults is False.

    Returns:
        None

    """
    data = {}
    if file_:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    if display:
        data["display"] = display
    return client._creoson_post("file", "regenerate", data)


def relations_get(client, file_=None):
    """Get relations for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (list:srt): Exported relations text, one entry per line.

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "relations_get", data, "relations")


def relations_set(client, file_=None, relations=None):
    """Set relations for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.
        relations (list:str, optional):
            Relations text to import, one line per entry.
            Clear the relations if missing.

    Returns:
        None

    """
    data = {}
    if file_:
        data["file"] = file_
    if relations:
        data["relations"] = relations
    return client._creoson_post("file", "relations_set", data)


def rename(client, new_name, file_=None, onlysession=None):
    """Rename a model.

    Args:
        client (obj):
            creopyson Client.
        new_name (str):
            New file name.
        `file_` (str, optional):
            File name. Defaults is currently active model.
        onlysession (boolean, optional):
            Modify only in memory, not on disk. Defaults is False.

    Returns:
        (str): The new model name.

    """
    data = {"new_name": new_name}
    if file_:
        data["file"] = file_
    if onlysession:
        data["onlysession"] = onlysession
    return client._creoson_post("file", "rename", data, "file")


def repaint(client, file_=None):
    """Repaint the window containing a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        None

    """
    data = {}
    if file_:
        data["file"] = file_
    return client._creoson_post("file", "repaint", data)


def save(client, file_):
    """Save one or more models.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str|list:str, optional):
            File name or List of file names;

    Returns:
        None

    """
    data = {}
    if file_:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    return client._creoson_post("file", "save", data)


def set_length_units(client, units, file_=None, convert=None):
    """Set the current length units for a model.

    This will search the model's available Unit Systems for the first one
    which contains the given length unit.

    Args:
        client (obj):
            creopyson Client.
        units (str):
            New length units.
        `file_` (str, optional):
            File name; only used if files is not given.
        convert (boolean, optional):
            Whether to convert the model's length values to the
            new units (True) or leave them the same value (False).
            Defaults is True.

    Returns:
        None

    """
    data = {"units": units}
    if file_:
        data["file"] = file_
    if convert:
        data["convert"] = convert
    return client._creoson_post("file", "set_length_units", data)


def set_mass_units(client, units, file_=None, convert=None):
    """Set the mass units for a model.

    This will search the model's available Unit Systems for the first one
    which contains the given mass unit.

    Args:
        client (obj):
            creopyson Client.
        units (str):
            New mass units.
        `file_` (str, optional):
            File name; only used if files is not given.
        convert (boolean, optional):
            Whether to convert the model's mass values to the
            new units (True) or leave them the same value (False).
            Defaults is True.

    Returns:
        None

    """
    data = {"units": units}
    if file_:
        data["file"] = file_
    if convert:
        data["convert"] = convert
    return client._creoson_post("file", "set_length_units", data)



"""Creo testing."""
import creopyson
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None


def test_creo_cd(mk_creoson_post_dict):
    """Test creo_cd."""
    c = creopyson.Client()
    dirname = "C:\\User\\Working directory\\"
    result = c.creo_cd(dirname)
    assert isinstance(result, (str))


def test_creo_delete_files(mk_creoson_post_dict):
    """Test creo_delete_files."""
    c = creopyson.Client()
    result = c.creo_delete_files()
    assert isinstance(result, (list))
    result = c.creo_delete_files(
        dirname="C:/dir/",
        filename="fake_file.prt")
    assert isinstance(result, (list))
    result = c.creo_delete_files(filename=["C:/dir/fake_file.prt"])
    assert isinstance(result, (list))


def test_creo_get_config(mk_creoson_post_dict):
    """Test creo_get_config."""
    c = creopyson.Client()
    result = c.creo_get_config("option")
    assert isinstance(result, (list))


def test_creo_get_std_color(mk_creoson_post_dict):
    """Test creo_get_std_color."""
    c = creopyson.Client()
    result = c.creo_get_std_color("letter")
    assert isinstance(result, (dict))


def test_creo_list_dirs(mk_creoson_post_dict):
    """Test creo_list_dirs."""
    c = creopyson.Client()
    result = c.creo_list_dirs("filter_*")
    assert isinstance(result, (list))


def test_creo_list_files(mk_creoson_post_dict):
    """Test creo_list_dirs."""
    c = creopyson.Client()
    result = c.creo_list_files("filter_*")
    assert isinstance(result, (list))


def test_creo_mkdir(mk_creoson_post_dict):
    """Test creo_mkdir."""
    c = creopyson.Client()
    result = c.creo_mkdir("new_dir")
    assert isinstance(result, (str))


def test_creo_pwd(mk_creoson_post_dict):
    """Test creo_pwd."""
    c = creopyson.Client()
    result = c.creo_pwd()
    assert isinstance(result, (str))


def test_creo_rmdir_ok(mk_creoson_post_None):
    """Test creo_rmdir."""
    c = creopyson.Client()
    result = c.creo_rmdir("fake_dir")
    assert result is None


def test_creo_set_config_ok(mk_creoson_post_None):
    """Test creo_set_config."""
    c = creopyson.Client()
    result = c.creo_set_config("option", 12, True)
    assert result is None


def test_creo_set_std_color_ok(mk_creoson_post_None):
    """Test creo_set_std_color."""
    c = creopyson.Client()
    result = c.creo_set_std_color("letter", 0, 0, 0)
    assert result is None



"""Feature testing."""
import creopyson
import pytest
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None, mk_creoson_post_list


def test_feature_delete(mk_creoson_post_None):
    """Test delete."""
    c = creopyson.Client()
    result = c.feature_delete(
        file_="file",
        name="name",
        status="ACTIVE",
        type_="type",
        clip=True
    )
    assert result is None
    result = c.feature_delete(
        name=["name1", "name2"]
    )
    assert result is None


def test_feature_delete_param(mk_creoson_post_None):
    """Test delete_param."""
    c = creopyson.Client()
    result = c.feature_delete_param(
        file_="file",
        name="name",
        param="param"
    )
    assert result is None


def test_feature_list(mk_creoson_post_list):
    """Test list."""
    c = creopyson.Client()
    result = c.feature_list(
        file_="file",
        name="name",
        type_="type",
        no_datum=True,
        inc_unnamed=True,
        no_comp=True,
        param="param",
        value="param",
        encoded=True
    )
    assert isinstance(result, (list))
    result = c.feature_list(
        param=["param", "other param"],
    )
    assert isinstance(result, (list))


def test_feature_param_exists(mk_creoson_post_dict):
    """Test param_exists."""
    c = creopyson.Client()
    result = c.feature_param_exists(
        file_="file",
        param="param"
    )
    assert result is True
    result = c.feature_param_exists(
        param=["param", "other param"]
    )
    assert result is True


def test_feature_rename(mk_creoson_post_None):
    """Test rename."""
    c = creopyson.Client()
    result = c.feature_rename(
        "oldname", "new_name",
        file_="file"
    )
    assert result is None
    result = c.feature_rename(
        1, "new_name",
        file_="file"
    )
    with pytest.raises(TypeError):
        c.feature_rename(
            ["list"], "new_name",
            file_="file"
        )


def test_feature_resume(mk_creoson_post_None):
    """Test resume."""
    c = creopyson.Client()
    result = c.feature_resume(
        file_="file",
        name="name",
        status="ACTIVE",
        type_="type",
        with_children=True
    )
    assert result is None
    result = c.feature_resume(
        name=["name", "other name"],
    )
    assert result is None


def test_feature_set_param(mk_creoson_post_None):
    """Test set_param."""
    c = creopyson.Client()
    result = c.feature_set_param(
        "param",
        file_="file",
        name="name",
        type_="STRING",
        value=12,
        encoded=True,
        designate=True,
        no_create=True
    )
    assert result is None


def test_feature_suppress(mk_creoson_post_None):
    """Test suppress."""
    c = creopyson.Client()
    result = c.feature_suppress(
        file_="file",
        name="name",
        status="ACTIVE",
        type_="type",
        clip=False,
        with_children=False
    )
    assert result is None
    result = c.feature_suppress(
        name=["name", "other name"],
    )
    assert result is None


def test_feature_user_select_csys(mk_creoson_post_list):
    """Test user_select_csys."""
    c = creopyson.Client()
    result = c.feature_user_select_csys(
        file_="file",
        max_=12
    )
    assert isinstance(result, (list))
